#!/bin/sh
src=`pwd`
tmp=$src/tests-tmp
rm -rf $tmp
mkdir -p $tmp
PATH="$src:/bin:/usr/bin:/usr/local/bin"
tests_failed=0
tests_count=0
_UID=`id -u`
_GID=`id -g`
sender=${tmp}/sender
receiver=${tmp}/receiver

KEYDIR=${tmp}
PORT=62144
SENDER=sender
SERVER=localhost
export KEYDIR PORT SENDER SERVER

cd ${tmp}

echo curve25519:6JH+h0pNEr7k9RoGCB+02RgGwtubhYkm1URwH0l0DWs= >curve25519
echo curve25519:TM7oIdoyfCO/n1Bsfj7+0wLULx33yhk0zEk3qRtUAEw= >curve25519.pub
echo nistp224:KlptWkgo8OVsRtvyJ/Y6hPsQpVoZUu7fby0+Qw== >nistp224
echo nistp224:kpthtYt+4wnPGGUkWNhzQo56QFY0PzDDdmv3Kw== >nistp224.pub

mkdir servers
echo curve25519:H466d963DtBcdtMZ6iN/A6WGu1vDbWuprp0s+khygCM= >servers/localhost.curve25519
echo nistp224:h7OoQ5jDDBW1wNZFss9AClWlOrMcZ1x+rxOrdg== >servers/localhost.nistp224

cleanup() {
  cd ${tmp}
  rm -rf receiver sender
}

setup() {
  cleanup

  mkdir receiver
  cd receiver
  echo sender:curve25519:TM7oIdoyfCO/n1Bsfj7+0wLULx33yhk0zEk3qRtUAEw= >senders
  echo sender:nistp224:kpthtYt+4wnPGGUkWNhzQo56QFY0PzDDdmv3Kw== >>senders
  echo curve25519:IC59FBExpUFvJB2ZeCEKB1nuYWt+DRyyxj/l5u2d00w= >secrets
  echo nistp224:aFXlMf5xs4f3kSAhKzfYLcXp6X6yAfqFJL9iLw== >>secrets
  chmod 444 senders
  cd ..

  mkdir sender
  cd sender
  echo 0:0 >sequence
  cd ..
}

startit() {
  setup
  cd ${tmp}/receiver
  ${src}/srlog2d -p $PORT >log 2>/dev/null &
  pid=$!
  cd ${tmp}/sender
}

stopit() {
  kill $pid
  sed -e 's/^@40000000................ //' \
  	${receiver}/log
  cleanup
  cd ${tmp}
}

run_compare_test() {
  local name=$1
  shift
  sed -e "s:@SOURCE@:$src:g"   	-e "s:@TMPDIR@:$tmp:g"   	-e "s:@UID@:$_UID:" 	-e "s:@GID@:$_GID:" 	>$tmp/expected
  ( runtest "$@" 2>&1 ) 2>&1 >$tmp/actual-raw
  cat -v $tmp/actual-raw >$tmp/actual
  if ! cmp $tmp/expected $tmp/actual >/dev/null 2>&1
  then
    echo "Test $name $@ failed:"
	( cd $tmp; diff -U 9999 expected actual | tail -n +3; echo; )
	tests_failed=$(($tests_failed+1))
  fi
  rm -f $tmp/expected $tmp/actual
  tests_count=$(($tests_count+1))
}

##### Test tests/xmit-buffer #####

runtest() {
startit
cat <<EOF >buffer
0 1134162292.530385500 0
1 1134162293.530385500 1
EOF
sleep 1 | ${src}/srlog2 service
stopit
}
run_compare_test tests/xmit-buffer  <<END_OF_TEST_RESULTS
:sender:service
0
1
END_OF_TEST_RESULTS


##### Test tests/xmit-skips #####

runtest() {
startit
cat <<EOF >buffer
0 1134162292.530385500 0
2 1134162293.530385500 2
EOF
sleep 1 | ${src}/srlog2 service
stopit
}
run_compare_test tests/xmit-skips  <<END_OF_TEST_RESULTS
:sender:service
0
2
END_OF_TEST_RESULTS


##### Test tests/xmit-stdin #####

runtest() {
startit
(
  echo 0
  echo 1
  sleep 1
) | ${src}/srlog2 service
stopit
}
run_compare_test tests/xmit-stdin  <<END_OF_TEST_RESULTS
:sender:service
0
1
END_OF_TEST_RESULTS


rm -rf $tmp
echo $tests_count tests executed, $tests_failed failures
if [ $tests_failed != 0 ]; then exit 1; fi
